from . import PromisePay

import json
import copy

class Wallet(PromisePay):
	
	def __init__(self,user_id):
		PromisePay.__init__(self)
		self.status='success'
		self.error_message = ''
		self.user_id= user_id
		
		self.g = self.get_one(path='/users/'+self.user_id+'/wallet_accounts/')
		
		print self.g
		if self.g.has_key('errors'):
			self.status='failed'
			self.error_message = self.g['errors']
		else:

			self.before_clean = copy.deepcopy(self.g)
			del self.g['wallet_accounts']['links']
			self.__dict__.update(json.loads(json.dumps(self.g['wallet_accounts'],sort_keys=True, indent=4)))
			self.__dict__.update(json.loads(json.dumps(self.before_clean['wallet_accounts']['links'],sort_keys=True, indent=4)))



	def deposit(self,account_id,amount):
		self.account_id= account_id
		self.amount = amount
		x = self.add_one(path='/wallet_accounts/'+self.id+'/deposit',data={'account_id':self.account_id,\
				'amount':str(int(self.amount))})

		if x.has_key('errors'):
			self.status='failed'
			try:
				self.error_message = x['errors']['transaction'][0]
			except:
				self.error_message = x['errors']

		return x


	def withdraw(self,target,amount):
		self.account_id= target
		self.amount = amount
		x = self.add_one(path='/wallet_accounts/'+self.id+'/withdraw',data={'account_id':self.account_id,\
				'amount':str(int(self.amount))})

		if x.has_key('errors'):
			self.status='failed'
			try:
				self.error_message = x['errors']['transaction'][0]
			except:
				self.error_message = x['errors']

		return x



class PromisePayCardAccount(PromisePay):
	
	def __init__(self,card_id=None):
		
		PromisePay.__init__(self)
		self.status='success'
		self.error_message = ''

		if card_id:
			self.card_id= card_id
			self.g = self.get_one(path='/card_accounts/'+self.card_id)
			
			if self.g.has_key('errors'):
				self.status='failed'
				self.error_message = self.g['errors']
			else:
				self.before_clean = copy.deepcopy(self.g)
				del self.g['card_accounts']['links']
				del self.g['card_accounts']['card']

				self.__dict__.update(json.loads(json.dumps(self.g['card_accounts'],sort_keys=True, indent=4)))
				self.__dict__.update(json.loads(json.dumps(self.before_clean['card_accounts']['links'],sort_keys=True, indent=4)))
				self.__dict__.update(json.loads(json.dumps(self.before_clean['card_accounts']['card'],sort_keys=True, indent=4)))


	def delete_card(self):
		self.g = self.delete_one(path='/card_accounts/',id=self.card_id)
		
		if self.g.has_key('errors'):
			self.status='failed'
			self.error_message = self.g['errors']
		

	def get_user_card(self,user_id):
		self.user_id= user_id
		self.g = self.get_one(path='/users/'+self.user_id+'/card_accounts/')
		
		if self.g.has_key('errors'):
			self.status='failed'
			self.error_message = self.g['errors']
		else:
			self.before_clean = copy.deepcopy(self.g)
			del self.g['card_accounts']['links']
			del self.g['card_accounts']['card']


			self.__dict__.update(json.loads(json.dumps(self.g['card_accounts'],sort_keys=True, indent=4)))
			self.__dict__.update(json.loads(json.dumps(self.before_clean['card_accounts']['links'],sort_keys=True, indent=4)))
			self.__dict__.update(json.loads(json.dumps(self.before_clean['card_accounts']['card'],sort_keys=True, indent=4)))



	def generate_token(self,user_id):

		promise_pay_token = ''

		try:
			k = self.add_one(path='token_auths',data={'token_type':'card','user_id':user_id})
			promise_pay_token = k["token_auth"]["token"]

			if k.has_key('errors'):
				self.status='failed'
				self.error_message = x['errors']
	
		except:
			self.status='failed'
			self.error_message = "Something went wrong"

		return promise_pay_token





class PromisePayBankAccount(PromisePay):
	
	def __init__(self,bank_id=None):
		
		PromisePay.__init__(self)
		self.status='success'
		self.error_message = ''

		if bank_id:
			self.bank_id= bank_id
			self.g = self.get_one(path='/bank_accounts/'+self.bank_id)
			
			if self.g.has_key('errors'):
				self.status='failed'
				self.error_message = self.g['errors']
			else:
				self.before_clean = copy.deepcopy(self.g)
				del self.g['bank_accounts']['links']
				del self.g['bank_accounts']['bank']

				self.__dict__.update(json.loads(json.dumps(self.g['bank_accounts'],sort_keys=True, indent=4)))
				self.__dict__.update(json.loads(json.dumps(self.before_clean['bank_accounts']['links'],sort_keys=True, indent=4)))
				self.__dict__.update(json.loads(json.dumps(self.before_clean['bank_accounts']['bank'],sort_keys=True, indent=4)))



	def get_user_bank(self,user_id):
		self.user_id= user_id
		self.g = self.get_one(path='/users/'+self.user_id+'/bank_accounts/')
		
		if self.g.has_key('errors'):
			self.status='failed'
			self.error_message = self.g['errors']
		else:
			self.before_clean = copy.deepcopy(self.g)
			del self.g['bank_accounts']['links']
			del self.g['bank_accounts']['bank']

			self.__dict__.update(json.loads(json.dumps(self.g['bank_accounts'],sort_keys=True, indent=4)))
			self.__dict__.update(json.loads(json.dumps(self.before_clean['bank_accounts']['links'],sort_keys=True, indent=4)))
			self.__dict__.update(json.loads(json.dumps(self.before_clean['bank_accounts']['bank'],sort_keys=True, indent=4)))

	
	def delete_bank(self):
		self.g = self.delete_one(path='/bank_accounts/',id=self.bank_id)
		
		if self.g.has_key('errors'):
			self.status='failed'
			self.error_message = self.g['errors']
		


	def generate_token(self,user_id):

		promise_pay_token = ''

		try:
			k = self.add_one(path='token_auths',data={'token_type':'bank','user_id':user_id})
			promise_pay_token = k["token_auth"]["token"]

			if k.has_key('errors'):
				self.status='failed'
				self.error_message = x['errors']
	
		except:
			self.status='failed'
			self.error_message = "Something went wrong"

		return promise_pay_token








class PromisePayUser(PromisePay):
	
	def __init__(self,user_id=None):
		PromisePay.__init__(self)
		self.status='success'
		self.error_message = ''
		
		if user_id:
			self.user_id= user_id
			self.g = self.get_one(path='/users/'+self.user_id)
			
			if self.g.has_key('errors'):
				self.status='failed'
				self.error_message = self.g['errors']
			
			else:
				self.before_clean = copy.deepcopy(self.g)
				del self.g['users']['links']
				del self.g['users']['related']
				self.__dict__.update(json.loads(json.dumps(self.g['users'],sort_keys=True, indent=4)))
				self.__dict__.update(json.loads(json.dumps(self.before_clean['users']['links'],sort_keys=True, indent=4)))


	def all(self,limit=10,offset=0):
		self.limit = limit
		self.offset=offset
		datas = self.get_list(path='/users/',limit=self.limit,offset=self.offset)['users']
		data_list = []

		for user in json.loads(json.dumps(datas,sort_keys=True, indent=4)):
			print user
			u = PromisePayUser(user['id'])
			data_list.append(u)

		return data_list


	def create(self,**kwargs):
		
		x = self.add_one(path='users',data=kwargs)
		
		if x.has_key('errors'):
			self.status='failed'
			self.error_message = x['errors']
		
		if self.status=='success':
			return PromisePayUser(x['id'])
		
		else:
			return self.error_message


	def update(self,**kwargs):

		if self.user_id:
			x = self.edit_one(path='users',id=self.user_id,data=kwargs)

			try:
				if x.json().has_key('errors'):
					self.status='failed'
					self.error_message = x.json()['errors']
				else:
					return PromisePayUser(self.user_id)
			
			except:
				self.status='failed'
				self.error_message = "Something went wrong"

			return x
		
		else:
			self.status='failed'
			self.error_message = 'User ID is required'






import requests
import base64
import json


import os


def auth(username,secret,is_production=False):
	os.environ['PROMISE_PAY_SECRET'] = secret
	os.environ['PROMISE_PAY_KEY'] = username 
	os.environ['is_production'] = str(is_production)
	



class PromisePay(object):


	
	def __init__(self):

		try:
			from django.conf import settings

			self.password = settings.PROMISE_PAY_SECRET
			self.username = settings.PROMISE_PAY_KEY

			self.is_production = getattr(settings, "PROMISE_PAY_IS_PROD", False)

			self.AUTH = 'Basic '+base64.b64encode(settings.PROMISE_PAY_KEY+':'+settings.PROMISE_PAY_SECRET)
			self.HEADERS = {'Authorization': self.AUTH,"Content-Type": "application/json"}


		except:

			if not os.environ.has_key('PROMISE_PAY_SECRET') or not os.environ.has_key('PROMISE_PAY_KEY'):
				raise Exception('Make sure you have imported auth from promisepay import auth , And set username and password secret ')

			self.username = os.environ.get('PROMISE_PAY_KEY')  
			self.password = os.environ.get('PROMISE_PAY_SECRET')  
			self.is_production = bool(os.environ.get('is_production'))

			self.AUTH = 'Basic '+base64.b64encode(self.username+':'+self.password)
			self.HEADERS = {'Authorization': self.AUTH,"Content-Type": "application/json"}

		if self.is_production:
			self.MASTER_URL = 'https://api.promisepay.com/'
		else:
			self.MASTER_URL = 'https://test.api.promisepay.com/'

			

	def get_list(self,path,limit=10,offset=0):

		self.limit = limit
		self.path=path
		self.offset=offset

		payload = {'limit': self.limit, 'offset': self.offset}

		r = requests.get(self.MASTER_URL+self.path, headers=self.HEADERS,params=payload)
		
		if r.json().has_key('meta'):
			return r.json()
		
		else:
			try:
				
				if r.status_code == 503:
					return {'errors':'Service is down'}
				else:
					return r.json()
			except:
				raise Exception(r.json())


	def get_one(self,path,id=None):
		

		if id:
			r = requests.get(self.MASTER_URL+path+'/'+id, headers=self.HEADERS)

		else:
			r = requests.get(self.MASTER_URL+path+'/', headers=self.HEADERS)
		
		if r.status_code == 503:
			return {'errors':'Service is down'}
		
		else:
			print r.json()
			return r.json()



	def delete_one(self,path,id=None):
		
		
		if id:
			r = requests.delete(self.MASTER_URL+path+'/'+id, headers=self.HEADERS)

		else:
			r = requests.delete(self.MASTER_URL+path+'/', headers=self.HEADERS)
		

		print "SCODE",r.status_code

		if r.status_code == 200:
			return r.json()

	
		if r.status_code == 503:
			return {'errors':'Service is down'}
		
		else:
			return {'errors':'Something went wrong with service provider'}
		



	def add_one(self,path,data=None):
	
		r = requests.post(self.MASTER_URL+path, headers=self.HEADERS,data=json.dumps(data))


		if r.status_code == 201:
			return r.json()

		elif r.status_code ==  401:
			raise Exception('Authorization error')


		if r.status_code == 503:
			return {'errors':'Service is down'}
		else:
			return r.json()


	def edit_one(self,path,id,data):

		r = requests.patch(self.MASTER_URL+path+'/'+id, data=json.dumps(data),headers=self.HEADERS)

		return r



from setuptools import setup
setup(name='promisepay',
      description='Python SDK for PromisePay/ Assemblypayments',
      version='1.4',
      url='https://github.com/Rekvant/promisepay-python',
      author='Anish Menon',
      author_email='info@rekvant.com',
      license='Apache2',
      classifiers=[
          'Development Status :: 4 - Beta',
          'Intended Audience :: System Administrators',
          'License :: OSI Approved :: Apache Software License',
          'Programming Language :: Python :: 2.7'
      ],

      packages=['promisepay'],
      install_requires=[
          'requests >=2.14',
      ],
     
)
